//---------------------------------------------------------------------
// CPPNActivationFunctions.h - CPPN activation functions
//						    - by Erin Hastings ejhastings@gmail.com
//
//	functions:	1==sin
//				2==cos
//				3==hyperbolic tangent
//				4==inverse
//				5==ramp
//				6==step
//				7==gaussian
//				8==spike
//				9==tangent
//			   10==increment (for testing only)
//			   11==identity (for testing only)
//---------------------------------------------------------------------
#ifndef CPPN_ACTIVATION_FUNCTIONS_H
#define CPPN_ACTIVATION_FUNCTIONS_H

//---------------------------------------------------------------------
// standard includes
//---------------------------------------------------------------------
#include <cmath>

//---------------------------------------------------------------------
// custom includes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// function 1: <cmath> sin
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// function 2: <cmath> cos
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// function 3: hyperbolic tangent <cmath> tanh 
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// function 4: inverse 
//---------------------------------------------------------------------
float inverse(float x)
{
	return -x;
}

//---------------------------------------------------------------------
// function 5: ramp  / / / / /
//---------------------------------------------------------------------
float ramp(float x)
{
	return 1.0f - 2.0f * (x-floor(x));
}

//---------------------------------------------------------------------
// function 6: step   -_-_-
//---------------------------------------------------------------------
float step(float x)
{
	if((int)floor(x)%2 >= 0.0f) return 1.0f;
	else return -1.0f;
}

//---------------------------------------------------------------------
// function 7: gaussian  ___/\___
//---------------------------------------------------------------------
float gaussian(float x)
{ 				
	return exp(-1.0f * (x * x));
}

//---------------------------------------------------------------------
// function 8: spike  /\/\/\
//---------------------------------------------------------------------
float spike(float x)
{	
	if((int)floor(x)%2 == 0) return 1.0f - 2.0f * (x-floor(x));
	else return -1.0f + 2.0f * (x-floor(x));
}

//---------------------------------------------------------------------
// function 9: tangent, adjusting for invalid return values
//---------------------------------------------------------------------
float tangent(float x)
{	
	x = tan(x);
	if(x<10000.0f && x>-10000.0f) return x;
	if(x > 10000.0f) return 10000.0f;
	if(x < -10000.0f) return -10000.0f;
	return 0.0f;
}

//---------------------------------------------------------------------
// function 10: increment (test function)
//---------------------------------------------------------------------
float increment(float x)
{		
	return x+1.0f;
}

//---------------------------------------------------------------------
// function 11: identity (test function)
//---------------------------------------------------------------------
float identity(float x)
{		
	return x;
}


#endif
