//---------------------------------------------------------------------
// CPPNNode.h - Node in an Artificial Neural Net
//			 - by Erin Hastings ejhastings@gmail.com
//---------------------------------------------------------------------
#ifndef CPPN_NODE_H
#define CPPN_NODE_H

//---------------------------------------------------------------------
// includes
//---------------------------------------------------------------------
#include "CPPNActivationFunctions.h"
#include "Random.h"
using namespace std;

//---------------------------------------------------------------------
// Node class - a Node in the neural network that contains data
//---------------------------------------------------------------------
class Node
{
	public:	

	// variables		
	float data;						
	float accumulator;	

	// pointer to activation function
	int activation_function;
	float (*function_ptr) (float);
	void assign_activation_function(int f);
	
	// constructors, destructors
	Node();							
	~Node();
};

//---------------------------------------------------------------------
// Node() - constructor
//---------------------------------------------------------------------
Node::Node()
{ 
	data=0.0f;
	accumulator=0.0f;

	// choose random activation function
	assign_activation_function(getRandomInt(1,9));
}

//---------------------------------------------------------------------
// ~Node() - destructor
//---------------------------------------------------------------------
Node::~Node() {}

//---------------------------------------------------------------------
// assign_activation_function() - assigns function_ptr to function
//---------------------------------------------------------------------
void Node::assign_activation_function(int f)
{
	activation_function = f;

	// assign the activation function pointer
	switch(f)
	{
		case 1:	function_ptr = &sin;
				break;

		case 2:	function_ptr = &cos;
				break;

		case 3:	function_ptr = &tanh;
				break;

		case 4:	function_ptr = &inverse;
				break;

		case 5:	function_ptr = &ramp;
				break;

		case 6:	function_ptr = &step;
				break;

		case 7:	function_ptr = &gaussian;
				break;

		case 8:	function_ptr = &spike;
				break;

		case 9:	function_ptr = &tangent;
				break;

		default:function_ptr = &identity;
	}
}

#endif