//---------------------------------------------------------------------
// FileIO.h - saving and loading functions
//---------------------------------------------------------------------
#ifndef FILE_IO_H
#define FILE_IO_H

//---------------------------------------------------------------------
// includes
//---------------------------------------------------------------------
#include <fstream>
using namespace std;

//---------------------------------------------------------------------
// extern variables
//---------------------------------------------------------------------
extern vector<CPPN> CPPN_VECTOR;		
extern vector<texture> TEXTURE_VECTOR;
extern int PARTICLE_CLASS;

//---------------------------------------------------------------------
// MakeDebugNote() - write simple note to file DEBUG_FILE.txt
//---------------------------------------------------------------------
void makeDebugNote(string note)
{
	// open file in append mode
	ofstream debugFile("DEBUG_FILE.txt", ios::app);

	// write note
	debugFile << note << endl << endl;

	// close file
	debugFile.close();
}

//---------------------------------------------------------------------
// Save_CPPN() - saves CPPN[index] to file CPPN_X.txt
//			  - X is saved in "save_files/particle_class/SAVE_FILE.txt"
//---------------------------------------------------------------------
void save_CPPN(int index)
{
	ofstream outputFile;
	ifstream inputFile;
	int save_file_number;
	stringstream ss;

	// find correct save file path based on particle class

	// generic
	if(PARTICLE_CLASS==0)
	{
		// find the current save file number
		inputFile.open("save_files/generic/SAVE_FILE.txt");
		inputFile >> save_file_number;
		inputFile.close();

		// increment save file number and save
		outputFile.open("save_files/generic/SAVE_FILE.txt");
		outputFile << save_file_number + 1;
		outputFile.close();

		// create outputFilename in format CPPN_X.txt
		ss << "save_files/generic/CPPN_";
		ss << save_file_number;
		ss << ".txt";
	}

	// beam
	else if(PARTICLE_CLASS==1)
	{
		// find the current save file number
		inputFile.open("save_files/beam/SAVE_FILE.txt");
		inputFile >> save_file_number;
		inputFile.close();

		// increment save file number and save
		outputFile.open("save_files/beam/SAVE_FILE.txt");
		outputFile << save_file_number + 1;
		outputFile.close();

		// create outputFilename in format CPPN_X.txt
		ss << "save_files/beam/CPPN_";
		ss << save_file_number;
		ss << ".txt";
	}

	// plane
	else if(PARTICLE_CLASS==2)
	{
		// find the current save file number
		inputFile.open("save_files/plane/SAVE_FILE.txt");
		inputFile >> save_file_number;
		inputFile.close();

		// increment save file number and save
		outputFile.open("save_files/plane/SAVE_FILE.txt");
		outputFile << save_file_number + 1;
		outputFile.close();

		// create outputFilename in format CPPN_X.txt
		ss << "save_files/plane/CPPN_";
		ss << save_file_number;
		ss << ".txt";
	}

	// rotator
	else if(PARTICLE_CLASS==3)
	{
		// find the current save file number
		inputFile.open("save_files/rotator/SAVE_FILE.txt");
		inputFile >> save_file_number;
		inputFile.close();

		// increment save file number and save
		outputFile.open("save_files/rotator/SAVE_FILE.txt");
		outputFile << save_file_number + 1;
		outputFile.close();

		// create outputFilename in format CPPN_X.txt
		ss << "save_files/rotator/CPPN_";
		ss << save_file_number;
		ss << ".txt";
	}

	// trail
	else if(PARTICLE_CLASS==4)
	{
		// find the current save file number
		inputFile.open("save_files/trail/SAVE_FILE.txt");
		inputFile >> save_file_number;
		inputFile.close();

		// increment save file number and save
		outputFile.open("save_files/trail/SAVE_FILE.txt");
		outputFile << save_file_number + 1;
		outputFile.close();

		// create outputFilename in format CPPN_X.txt
		ss << "save_files/trail/CPPN_";
		ss << save_file_number;
		ss << ".txt";
	}

	// save the CPPN with correct file name
	CPPN_VECTOR[index].save_to_file(ss.str());
}

//---------------------------------------------------------------------
// load_CPPN()  - loads a CPPN[index] from file
//			   - files stored in "load_files/particle_class/CPPN_X.txt" 
//---------------------------------------------------------------------
void load_CPPN(int index)
{
	// local variables
	ifstream inputFile;
	stringstream ss;

	// find correct load file path based on particle class

	// generic
	if(PARTICLE_CLASS==0)
	{
		ss << "load_files/generic/CPPN_";
		ss << index;
		ss << ".txt";
	}

	// beam
	else if(PARTICLE_CLASS==1)
	{
		ss << "load_files/beam/CPPN_";
		ss << index;
		ss << ".txt";
	}

	// plane
	else if(PARTICLE_CLASS==2)
	{
		ss << "load_files/plane/CPPN_";
		ss << index;
		ss << ".txt";
	}

	// rotator
	else if(PARTICLE_CLASS==3)
	{
		ss << "load_files/rotator/CPPN_";
		ss << index;
		ss << ".txt";
	}

	// trail
	else if(PARTICLE_CLASS==4)
	{
		ss << "load_files/trail/CPPN_";
		ss << index;
		ss << ".txt";
	}

	// load the CPPN file
	CPPN_VECTOR[index].load_from_file(ss.str());
}

//---------------------------------------------------------------------
// load_all() - load all CPPN from file
//			  - occurs on class change
//---------------------------------------------------------------------
void load_all()
{
	int i;

	// load CPPNs
	for(i=0; i<CPPN_VECTOR.size(); i++)
	{
		load_CPPN(i);
	}
}

#endif