//----------------------------------------------------------------------------
// Font. h	BuildFont - builds open GL font list
//			KillFont - destroys font list before exit
//			glPrint - prints on screen text
//----------------------------------------------------------------------------
#ifndef FONT_H
#define FONT_H

//----------------------------------------------------------------------------
// includes
//----------------------------------------------------------------------------
#include "NeHeGL.h"							 

//----------------------------------------------------------------------------
// function declarations
//----------------------------------------------------------------------------
void BuildFont(GLuint &base);					 // builds gl font list
void KillFont(GLuint &base);					 // Delete The Font List
void glPrint(GLuint &base, const char *fmt, ...);// Custom GL "Print" Routine

//----------------------------------------------------------------------------
//  externs
//----------------------------------------------------------------------------
extern GL_Window*	g_window;					 // declared in Globals.h
extern GLuint FONT_BASEe;						 // in main.cpp

//----------------------------------------------------------------------------
// Build Font -  OpenGL font support
//----------------------------------------------------------------------------
void BuildFont(GLuint &base)							// Build  Bitmap Font
{
	HFONT	font;										// Windows Font ID
	HFONT	oldfont;									// Used For Good House Keeping

	base = glGenLists(96);								// Storage For 96 Characters

	font = CreateFont(	16,								// Height
						0,								// Width
						0,								// Angle Of Escapement
						0,								// Orientation Angle
						FW_BOLD,						// Weight
						FALSE,							// Italic
						FALSE,							// Underline
						FALSE,							// Strikeout
						ANSI_CHARSET,					// Character Set 
						OUT_TT_PRECIS,					// Output Precision
						CLIP_DEFAULT_PRECIS,			// Clipping Precision
						ANTIALIASED_QUALITY,			// Output Quality
						FF_DONTCARE|DEFAULT_PITCH,		// Family | Pitch
						"Verdana");						// Name

	oldfont = (HFONT)SelectObject(g_window->hDC, font);	// Selects The Font We Want
	wglUseFontBitmaps(g_window->hDC, 32, 96, base);		// Builds 96 Characters Starting At Character 32
	SelectObject(g_window->hDC, oldfont);				// Selects The Font We Want
	DeleteObject(font);									// Delete The Font
}

//----------------------------------------------------------------------------
// Kill Font
//----------------------------------------------------------------------------
void KillFont(GLuint &base)								
{
	glDeleteLists(base, 96);								
}

//----------------------------------------------------------------------------
// glPrint - print font to window
//----------------------------------------------------------------------------

void glPrint(GLuint &base, const char *fmt, ...)	
{
	char		text[512];							// Holds Our String
	va_list		ap;									// Pointer To List Of Arguments

	if (fmt == NULL) return;						// if no text do nothing

	va_start(ap, fmt);								// Parses The String For Variables
	vsprintf(text, fmt, ap);						// And Converts Symbols To Actual Numbers
	va_end(ap);										// Results Are Stored In Text

	glPushAttrib(GL_LIST_BIT);						// Pushes The Display List Bits
	glListBase(base - 32);							// Sets The Base Character to 32
	glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);// Draws The Display List Text
	glPopAttrib();									// Pops The Display List Bits
}

#endif