//-------------------------------------------------------------
// plane_particle.h:  a single plane in a plane particle system
//-------------------------------------------------------------

#ifndef PLANE_PARTICLE_H
#define PLANE_PARTICLE_H

//-------------------------------------------------------------
// includes / defines
//-------------------------------------------------------------
#include <d3dx9.h>					// DirectX 9

//-------------------------------------------------------------
// particle class - a single particle in 3d space
//-------------------------------------------------------------
class plane_particle
{
	public:

	D3DXVECTOR3 position_A;			// position of corner A
	D3DXVECTOR3 velocity_A;			// velocity of corner A
	D3DXVECTOR3 color_A;			// color of corner A

	D3DXVECTOR3 position_B;			// position of corner B
	D3DXVECTOR3 velocity_B;			// velocity of corner B
	D3DXVECTOR3 color_B;			// color of corner B

	D3DXVECTOR3 position_C;			// position of corner C
	D3DXVECTOR3 velocity_C;			// velocity of corner C
	D3DXVECTOR3 color_C;			// color of corner C

	D3DXVECTOR3 position_D;			// position of corner D
	D3DXVECTOR3 velocity_D;			// velocity of corner D
	D3DXVECTOR3 color_D;			// color of corner D


	float time_to_live;				// lifespan
	float time;						// time since spawn		
	bool active;					// is this particle active

	plane_particle();				// default contstructor
	~plane_particle();
	plane_particle(	float px_a, float py_a, float pz_a,
					float vx_a, float vy_a, float vz_a,
					float cr_a, float cg_a, float cb_a,
					float px_b, float py_b, float pz_b,
					float vx_b, float vy_b, float vz_b,
					float cr_b, float cg_b, float cb_b,
					float px_c, float py_c, float pz_c,
					float vx_c, float vy_c, float vz_c,
					float cr_c, float cg_c, float cb_c,
					float px_d, float py_d, float pz_d,
					float vx_d, float vy_d, float vz_d,
					float cr_d, float cg_d, float cb_d,
					float ttl); 
};

//-------------------------------------------------------------
// default constructor
//-------------------------------------------------------------
plane_particle::plane_particle()
{
	position_A.x = 1.0f;	// position of A
	position_A.y = 0.0f;	
	position_A.z = 1.0f;	
	velocity_A.x = 1.0f;	// velocity of A
	velocity_A.y = 1.0f;	
	velocity_A.z = 1.0f;
	color_A.x = 1.0f;		// color of A
	color_A.y = 0.0f;	
	color_A.z = 1.0f;	

	position_B.x = 1.0f;	// position of B
	position_B.y = 0.0f;	
	position_B.z = -1.0f;	
	velocity_B.x = 0.0f;	// velocity of B
	velocity_B.y = 1.0f;	
	velocity_B.z = 0.0f;
	color_B.x = 0.0f;		// color of B
	color_B.y = 0.0f;	
	color_B.z = 1.0f;	

	position_C.x = -1.0f;	// position of C
	position_C.y = 0.0f;	
	position_C.z = -1.0f;	
	velocity_C.x = 0.0f;	// velocity of C
	velocity_C.y = 1.0f;	
	velocity_C.z = 0.0f;
	color_C.x = 0.0f;		// color of C
	color_C.y = 1.0f;	
	color_C.z = 0.0f;	

	position_D.x = -1.0f;	// position of D
	position_D.y = 0.0f;	
	position_D.z = 1.0f;	
	velocity_D.x = 0.0f;	// velocity of D
	velocity_D.y = 1.0f;	
	velocity_D.z = 0.0f;
	color_D.x = 0.0f;		// color of D
	color_D.y = 0.0f;	
	color_D.z = 1.0f;	

	time_to_live = 5.0f;
	time = 0.0f;
	active = false;
}

//-------------------------------------------------------------
// destructor
//-------------------------------------------------------------
plane_particle::~plane_particle()
{}

//-------------------------------------------------------------
// custom contstructor
//-------------------------------------------------------------
plane_particle::plane_particle(	float px_a, float py_a, float pz_a,
								float vx_a, float vy_a, float vz_a,
								float cr_a, float cg_a, float cb_a,
								float px_b, float py_b, float pz_b,
								float vx_b, float vy_b, float vz_b,
								float cr_b, float cg_b, float cb_b,
								float px_c, float py_c, float pz_c,
								float vx_c, float vy_c, float vz_c,
								float cr_c, float cg_c, float cb_c,
								float px_d, float py_d, float pz_d,
								float vx_d, float vy_d, float vz_d,
								float cr_d, float cg_d, float cb_d,
								float ttl) 
{
	position_A.x = px_a;	// position of A
	position_A.y = py_a;	
	position_A.z = pz_a;	
	velocity_A.x = vx_a;	// velocity of A
	velocity_A.y = vy_a;	
	velocity_A.z = vz_a;
	color_A.x = cr_a;		// color of A
	color_A.y = cg_a;	
	color_A.z = cb_a;	

	position_B.x = px_b;	// position of B
	position_B.y = py_b;	
	position_B.z = pz_b;	
	velocity_B.x = vx_b;	// velocity of B
	velocity_B.y = vy_b;	
	velocity_B.z = vz_b;
	color_B.x = cr_b;		// color of B
	color_B.y = cg_b;	
	color_B.z = cb_b;	

	position_C.x = px_c;	// position of C
	position_C.y = py_c;	
	position_C.z = pz_c;	
	velocity_C.x = vx_c;	// velocity of C
	velocity_C.y = vy_c;	
	velocity_C.z = vz_c;
	color_C.x = cr_c;		// color of C
	color_C.y = cg_c;	
	color_C.z = cb_c;	

	position_D.x = px_d;	// position of D
	position_D.y = py_d;	
	position_D.z = pz_d;	
	velocity_D.x = vx_d;	// velocity of D
	velocity_D.y = vy_d;	
	velocity_D.z = vz_d;
	color_D.x = cr_d;		// color of D
	color_D.y = cg_d;	
	color_D.z = cb_d;	

	time_to_live = ttl;
	time = 0.0f;
	active = false;
}	

#endif