//-------------------------------------------------------------
// particle_system.h:  particle_system class - a group of particles
//-------------------------------------------------------------

#ifndef PLANE_SYSTEM_H
#define PLANE_SYSTEM_H

//-------------------------------------------------------------
// includes / defines
//-------------------------------------------------------------

#include <d3dx9.h>				// DirectX 9
#include <vector>				// vector class
#include "PlaneParticle.h"		// particle class
#include "Random.h"				// random number functions
using namespace std;

//-------------------------------------------------------------
// particle_system - a single particle in 3d space
//-------------------------------------------------------------
class plane_system
{
	public:

	D3DXVECTOR3 position;					// position
	vector<plane_particle> particle_vector;	// list of particles
	float counter;							// for next particle release
	float frequency;						// how often new particles
	float system_life;						// how long system has been alive
	float system_ttl;						// how long system lasts

	plane_system();							// default contructor
	~plane_system();						// destructor

	void update(float time_elapsed, int index);			
	void draw();
	void reset(int index);

};

//-------------------------------------------------------------
// particle_system::particle_system() - constructor
//-------------------------------------------------------------
plane_system::plane_system()
{	
	position.x = 0.0f;
	position.y = 0.0f;
	position.z = 0.0f;

	counter =0.0f;
	frequency = 1.0f;
	system_ttl = 5.0f;
	system_life = 0.0f;

	int i;
	for(i=0; i<20; i++)
	{
		plane_particle q;
		particle_vector.push_back(q);
	}
}

//-------------------------------------------------------------
// particle_system::~particle_system() - destructor
//-------------------------------------------------------------
plane_system::~plane_system()
{}

//-------------------------------------------------------------
// particle_system::update()
//-------------------------------------------------------------
void plane_system::update(float time_elapsed, int index)
{	
	int i;

	// update lifespan
	system_life+=time_elapsed;

	// check if lifespan is done
	if(system_life > system_ttl)
	{
		// reset system
		for(i=0; i<particle_vector.size(); i++)
		{
			particle_vector[i].active=false;
		}

		system_life = 0.0f;
	}

	// update particle timers
	for(i=0; i<particle_vector.size(); i++)
	{
		// only check active particles
		if(particle_vector[i].active)
		{
			// update timer
			particle_vector[i].time+=time_elapsed;

			// deativate if time expired
			if(particle_vector[i].time>particle_vector[i].time_to_live)
			{
				particle_vector[i].active=false;
			}
		}
	}

	// check to add new particle
	counter+=time_elapsed;
	if(counter>frequency)
	{
		// find an inactive particle
		for(i=0; i<particle_vector.size(); i++)
		{
			// activate it
			if(!particle_vector[i].active)
			{
				particle_vector[i].active=true;
				
				// position of A
				particle_vector[i].position_A.x = getRandomFloat(0.5f, 1.0f);
				particle_vector[i].position_A.y = 0.0f;	
				particle_vector[i].position_A.z = getRandomFloat(0.5f, 1.0f);	

				// velocity of A
				particle_vector[i].velocity_A.x = 1.0f;	
				particle_vector[i].velocity_A.y = 0.05f;	
				particle_vector[i].velocity_A.z = 1.0f;

				// color of A
				particle_vector[i].color_A.x = 0.0f;	
				particle_vector[i].color_A.y = 0.0f;	
				particle_vector[i].color_A.z = 0.0f;	

				// position of B
				particle_vector[i].position_B.x = getRandomFloat(0.5f, 1.0f);	
				particle_vector[i].position_B.y = 0.0f;	
				particle_vector[i].position_B.z = -1.0f + getRandomFloat(0.0f, 0.5f);

				// velocity of B
				particle_vector[i].velocity_B.x = 1.0f;	
				particle_vector[i].velocity_B.y = 0.05f;	
				particle_vector[i].velocity_B.z = -1.0f;

				// color of B
				particle_vector[i].color_B.x = 0.0f;	
				particle_vector[i].color_B.y = 0.0f;	
				particle_vector[i].color_B.z = 0.0f;	

				// position of C
				particle_vector[i].position_C.x = -1.0f + getRandomFloat(0.0f, 0.5f);
				particle_vector[i].position_C.y = 0.0f;	
				particle_vector[i].position_C.z = -1.0f + getRandomFloat(0.0f, 0.5f);

				// velocity of C
				particle_vector[i].velocity_C.x = -1.0f;	
				particle_vector[i].velocity_C.y = 0.05f;
				particle_vector[i].velocity_C.z = -1.0f;

				// color of C
				particle_vector[i].color_C.x = 0.0f;	
				particle_vector[i].color_C.y = 0.0f;	
				particle_vector[i].color_C.z = 0.0f;	

				// position of D
				particle_vector[i].position_D.x = -1.0f + getRandomFloat(0.0f, 0.5f);
				particle_vector[i].position_D.y = 0.0f;	
				particle_vector[i].position_D.z = getRandomFloat(0.5f, 1.0f);	

				// velocity of D
				particle_vector[i].velocity_D.x = -1.0f;	
				particle_vector[i].velocity_D.y = 0.05f;	
				particle_vector[i].velocity_D.z = 1.0f;

				// color of D
				particle_vector[i].color_D.x = 0.0f;	
				particle_vector[i].color_D.y = 0.0f;	
				particle_vector[i].color_D.z = 0.0f;	

				particle_vector[i].time_to_live = 5.0f;
				particle_vector[i].time = 0.0f;

				// reset counter and break
				counter = 0.0f;
				break;
			}
		}
	}

	// update all particle positions
	for(i=0; i<particle_vector.size(); i++)
	{
		// inputs = position/DFC 
		// in: 0=px, 1=py, 2=pz, 3=dfc
		// out: 4=vx, 5=vy, 6=vz, 7=r, 8=g, 9=b
		if(particle_vector[i].active)
		{
			// compute corner A	+x +z -------------------------------------------------
			// plug corner A postion into CPPN
			CPPN_VECTOR[index].nodes[0].data = particle_vector[i].position_A.x; 
			CPPN_VECTOR[index].nodes[1].data = particle_vector[i].position_A.y; 
			CPPN_VECTOR[index].nodes[2].data = particle_vector[i].position_A.z; 

			// plug corner A DFC into CPPN
			CPPN_VECTOR[index].nodes[3].data = D3DXVec3Length(&particle_vector[i].position_A);

			// activate neural net
			CPPN_VECTOR[index].activate();

			// get corner A color from CPPN
			particle_vector[i].color_A.x = abs(CPPN_VECTOR[index].nodes[4].data);
			particle_vector[i].color_A.y = abs(CPPN_VECTOR[index].nodes[5].data);
			particle_vector[i].color_A.z = abs(CPPN_VECTOR[index].nodes[6].data);

			// get corner A velocity +x +z
			particle_vector[i].velocity_A.x = abs(CPPN_VECTOR[index].nodes[7].data);
			particle_vector[i].velocity_A.y = 0.05f;
			particle_vector[i].velocity_A.z = abs(CPPN_VECTOR[index].nodes[9].data);

			// update corner A position
			particle_vector[i].position_A += particle_vector[i].velocity_A * 
											 time_elapsed * ACCELERATION;	

			// compute corner B +x -z -----------------------------------------------	
			// plug corner B postion into CPPN
			CPPN_VECTOR[index].nodes[0].data = particle_vector[i].position_B.x; 
			CPPN_VECTOR[index].nodes[1].data = particle_vector[i].position_B.y; 
			CPPN_VECTOR[index].nodes[2].data = particle_vector[i].position_B.z; 

			// plug corner B DFC into CPPN
			CPPN_VECTOR[index].nodes[3].data = D3DXVec3Length(&particle_vector[i].position_B);

			// activate neural net
			CPPN_VECTOR[index].activate();

			// get corner B color from CPPN
			particle_vector[i].color_B.x = abs(CPPN_VECTOR[index].nodes[4].data);
			particle_vector[i].color_B.y = abs(CPPN_VECTOR[index].nodes[5].data);
			particle_vector[i].color_B.z = abs(CPPN_VECTOR[index].nodes[6].data);

			// get corner B velocity +x +z
			particle_vector[i].velocity_B.x = abs(CPPN_VECTOR[index].nodes[7].data);
			particle_vector[i].velocity_B.y = 0.05f;
			particle_vector[i].velocity_B.z = -1.0f * abs(CPPN_VECTOR[index].nodes[9].data);

			// update corner B position
			particle_vector[i].position_B += particle_vector[i].velocity_B * 
											 time_elapsed * ACCELERATION;

			// compute corner C -x -z -----------------------------------------------	
			// plug corner C postion into CPPN
			CPPN_VECTOR[index].nodes[0].data = particle_vector[i].position_C.x; 
			CPPN_VECTOR[index].nodes[1].data = particle_vector[i].position_C.y; 
			CPPN_VECTOR[index].nodes[2].data = particle_vector[i].position_C.z; 

			// plug corner C DFC into CPPN
			CPPN_VECTOR[index].nodes[3].data = D3DXVec3Length(&particle_vector[i].position_C);

			// activate neural net
			CPPN_VECTOR[index].activate();

			// get corner C color from CPPN
			particle_vector[i].color_C.x = abs(CPPN_VECTOR[index].nodes[4].data);
			particle_vector[i].color_C.y = abs(CPPN_VECTOR[index].nodes[5].data);
			particle_vector[i].color_C.z = abs(CPPN_VECTOR[index].nodes[6].data);

			// get corner C velocity +x +z
			particle_vector[i].velocity_C.x = -1.0f *abs(CPPN_VECTOR[index].nodes[7].data);
			particle_vector[i].velocity_C.y = 0.05f;
			particle_vector[i].velocity_C.z = -1.0f * abs(CPPN_VECTOR[index].nodes[9].data);

			// update corner C position
			particle_vector[i].position_C += particle_vector[i].velocity_C * 
											 time_elapsed * ACCELERATION;

			// compute corner C -x +z -----------------------------------------------	
			// plug corner D postion into CPPN
			CPPN_VECTOR[index].nodes[0].data = particle_vector[i].position_D.x; 
			CPPN_VECTOR[index].nodes[1].data = particle_vector[i].position_D.y; 
			CPPN_VECTOR[index].nodes[2].data = particle_vector[i].position_D.z; 

			// plug corner D DFC into CPPN
			CPPN_VECTOR[index].nodes[3].data = D3DXVec3Length(&particle_vector[i].position_D);

			// activate neural net
			CPPN_VECTOR[index].activate();

			// get corner D color from CPPN
			particle_vector[i].color_D.x = abs(CPPN_VECTOR[index].nodes[4].data);
			particle_vector[i].color_D.y = abs(CPPN_VECTOR[index].nodes[5].data);
			particle_vector[i].color_D.z = abs(CPPN_VECTOR[index].nodes[6].data);

			// get corner D velocity +x +z
			particle_vector[i].velocity_D.x = -1.0f *abs(CPPN_VECTOR[index].nodes[7].data);
			particle_vector[i].velocity_D.y = 0.05f;
			particle_vector[i].velocity_D.z = abs(CPPN_VECTOR[index].nodes[9].data);

			// update corner D position
			particle_vector[i].position_D += particle_vector[i].velocity_D * 
											 time_elapsed * ACCELERATION;
		}
	}
}

//-------------------------------------------------------------
// particle_system::draw()
//-------------------------------------------------------------
void plane_system::draw()
{
	unsigned int i;

	// iterate through particle list ...
	for(i=0; i<particle_vector.size(); i++)		
	{	
		if(particle_vector[i].active)
		{
			// render billboard
			glBegin(GL_TRIANGLE_STRIP);
				glTexCoord2d(1,1);
				glColor3fv(particle_vector[i].color_C);	
				glVertex3fv(particle_vector[i].position_C);

				glTexCoord2d(0,1);
				glColor3fv(particle_vector[i].color_D);	
				glVertex3fv(particle_vector[i].position_D);

				glTexCoord2d(1,0);
				glColor3fv(particle_vector[i].color_B);	
				glVertex3fv(particle_vector[i].position_B);

				glTexCoord2d(0,0);
				glColor3fv(particle_vector[i].color_A);	
				glVertex3fv(particle_vector[i].position_A);
			glEnd();
		}
	}
}

//-------------------------------------------------------------
// particle_system::reset() - resets to simple system
//-------------------------------------------------------------
void plane_system::reset(int index)
{
	// reset CPPN
	CPPN_VECTOR[index].reset(4,6);
}

#endif