//-------------------------------------------------------------
// particle_system.h:  particle_system class - a group of particles
//-------------------------------------------------------------

#ifndef ROTATOR_SYSTEM_H
#define ROTATOR_SYSTEM_H

//-------------------------------------------------------------
// includes / defines
//-------------------------------------------------------------	
#include "RotatorParticle.h"	

//-------------------------------------------------------------
// rotator_system - a single particle in 3d space
//-------------------------------------------------------------
class rotator_system
{
	public:

	D3DXVECTOR3 position;						// position
	vector<rotator_particle> particle_vector;	// list of particles
	float frequency;							// how often new particles spawn
	float counter;								// for next particle release

	rotator_system();						
	~rotator_system();						

	void update(float time_elapsed, int index);			
	void draw();
	void reset(int index);

};

//-------------------------------------------------------------
// rotator_system::rotator_system() - constructor
//-------------------------------------------------------------
rotator_system::rotator_system()
{
	int i;

	position.x=0.0f;
	position.y=0.0f;
	position.z=0.0f;
	counter=0.0f;
	frequency=0.2f;

	int random_int;

	// add initial particles
	for(i=0; i<PARTICLE_DENSITY; i++)
	{

		// set some random active particles
		random_int=getRandomInt(1,20);

		// create particle
		rotator_particle r;
		if(random_int<10)
		{
			r.position.x = 0.0f;
			r.position.y = 0.0f;
			r.position.z = 0.0f;
			r.color.x = 0.0f;
			r.color.y = 0.0f;
			r.color.z = 0.0f;
			r.time=0.0f;
			r.time_to_live=getRandomFloat(0.0f, 5.0f);
			r.active = true;
		}
		else r.active = false;

		particle_vector.push_back(r);
	}
}

//-------------------------------------------------------------
// rotator_system::~rotator_system() - destructor
//-------------------------------------------------------------
rotator_system::~rotator_system()
{}

//-------------------------------------------------------------
// rotator_system::update()
//-------------------------------------------------------------
void rotator_system::update(float time_elapsed, int index)
{	
	int i;

	// update particle timers
	for(i=0; i<particle_vector.size(); i++)
	{
		if(particle_vector[i].active)
		{
			particle_vector[i].time +=time_elapsed;

			// check for particles expiration
			if(particle_vector[i].time > particle_vector[i].time_to_live)
			{
				// deactivate particle	
				particle_vector[i].active = false;
			}
		}
	}

	// update new particle counter
	counter+=time_elapsed;

	// check to add new particles
	if(counter>frequency)
	{
		// find an inactive particle and activate it
		for(i=0; i<particle_vector.size(); i++)
		{
			if(!particle_vector[i].active)
			{
				// reset particle position
				switch(GENERATION_SHAPE)
				{
					// tight cloud
					case 0:	particle_vector[i].position.x= -3.0f + getRandomFloat( 0.0f, 6.0f);  // pos x
							particle_vector[i].position.y= -3.0f + getRandomFloat( 0.0f, 6.0f);  // pos y
							particle_vector[i].position.z= -3.0f + getRandomFloat( 0.0f, 6.0f);  // pos z 
							break;

					// point
					case 1:	particle_vector[i].position.x= -0.2f + getRandomFloat( 0.0f, 0.4f);  // pos x
							particle_vector[i].position.y= -0.2f + getRandomFloat( 0.0f, 0.4f);  // pos y
							particle_vector[i].position.z= -0.2f + getRandomFloat( 0.0f, 0.4f);  // pos z 
							break;

					// line
					case 2:	particle_vector[i].position.x= position.x;  // pos x
							particle_vector[i].position.y= position.y;  // pos y
							particle_vector[i].position.z= -4.0f + getRandomFloat( 0.0f, 8.0f);  // pos z 
							break;

					// circle
					case 3:	particle_vector[i].position.x= -3.0f + getRandomFloat( 0.0f, 6.0f);  // pos x
							particle_vector[i].position.y= position.y;							 // pos y
							particle_vector[i].position.z= -3.0f + getRandomFloat( 0.0f, 6.0f);  // pos z 
							D3DXVec3Normalize(&particle_vector[i].position, &particle_vector[i].position);
							particle_vector[i].position*=3.0f;
							break;
				
					// wide cloud
					case 4:	particle_vector[i].position.x= -4.0f + getRandomFloat( 0.0f, 8.0f);  // pos x
							particle_vector[i].position.y= -4.0f + getRandomFloat( 0.0f, 8.0f);  // pos y
							particle_vector[i].position.z= -4.0f + getRandomFloat( 0.0f, 8.0f);  // pos z 
							break;
				
					// field
					case 5:	particle_vector[i].position.x= -10.0f + getRandomFloat( 0.0f, 20.0f);  // pos x
							particle_vector[i].position.y= -10.0f + getRandomFloat( 0.0f, 20.0f);  // pos y
							particle_vector[i].position.z= -10.0f + getRandomFloat( 0.0f, 20.0f);  // pos z 
				}

				// load CPPN
				// inputs: px, py, pz, dfc
				// outputs: rx, ry, rz, r, g, b
				CPPN_VECTOR[index].nodes[0].data = particle_vector[i].position.x;
				CPPN_VECTOR[index].nodes[1].data = particle_vector[i].position.y;
				CPPN_VECTOR[index].nodes[2].data = particle_vector[i].position.z;
				CPPN_VECTOR[index].nodes[3].data = D3DXVec3Length(&particle_vector[i].position);

				// activate CPPN
				CPPN_VECTOR[index].activate();

				// get particle rotation amount from CPPN outputs
				particle_vector[i].x_increment = CPPN_VECTOR[index].nodes[4].data;
				particle_vector[i].y_increment = CPPN_VECTOR[index].nodes[5].data;
				particle_vector[i].z_increment = CPPN_VECTOR[index].nodes[6].data;

				// get particle color from CPPN outputs
				particle_vector[i].color.x = CPPN_VECTOR[index].nodes[7].data;
				particle_vector[i].color.y = CPPN_VECTOR[index].nodes[8].data;
				particle_vector[i].color.z = CPPN_VECTOR[index].nodes[9].data;

				// reset particle time,ttl
				particle_vector[i].time=0.0f;
				particle_vector[i].time_to_live=getRandomFloat(2.0f, 5.0f);

				// activate particle and break
				particle_vector[i].active = true;
				break;
			}
		}

		// reset the counter
		counter=0.0f;
	}
	
	// time elapsed isnt quite fast enough
	time_elapsed *= 10.0f;

	// update each rotator particle
	for(i=0; i<particle_vector.size(); i++)
	{
		if(particle_vector[i].active)
		{
			particle_vector[i].size = 1.0f - particle_vector[i].time/particle_vector[i].time_to_live; 
			particle_vector[i].angle_x += particle_vector[i].x_increment * time_elapsed * ACCELERATION;
			particle_vector[i].angle_y += particle_vector[i].y_increment * time_elapsed * ACCELERATION;
			particle_vector[i].angle_z += particle_vector[i].z_increment * time_elapsed * ACCELERATION;
		}
	}
}

//-------------------------------------------------------------
// rotator_system::draw()
//-------------------------------------------------------------
void rotator_system::draw()
{
	int i,j;
	D3DXVECTOR3 A,B,C,D;
	float angle_increment = 0.0f;
	float size_increment = 0.0f;

	// draw each rotator particle
	for(i=0; i<particle_vector.size(); i++)
	{
		// only draw active particles
		if(particle_vector[i].active)
		{
			// set color
			glColor3f(particle_vector[i].color.x,			
					  particle_vector[i].color.y,
					  particle_vector[i].color.z);	

			angle_increment = -5.0f;
			size_increment = -0.05f;

			for(j=0; j<10; j++)
			{
				angle_increment += 5.0f;
				size_increment +=0.05f;

				if(particle_vector[i].size - size_increment > 0)
				{
					glPushMatrix();

					// set proper rotation
					if(particle_vector[i].x_increment > 0.0f)
						glRotatef(particle_vector[i].angle_x - angle_increment, 1.0f, 0.0f, 0.0f);
					if(particle_vector[i].y_increment > 0.0f)
						glRotatef(particle_vector[i].angle_y - angle_increment, 0.0f, 1.0f, 0.0f);
					if(particle_vector[i].z_increment > 0.0f)
						glRotatef(particle_vector[i].angle_z - angle_increment, 0.0f, 0.0f, 1.0f);

					// setup correct billboards
					SetupBillboards(RIGHT, UP);

					// compute corners as vectors A,B,C,D
					A = particle_vector[i].position -				
						(RIGHT+UP) *
						(particle_vector[i].size - size_increment);	
					B = particle_vector[i].position +
						(RIGHT-UP) *
						(particle_vector[i].size - size_increment);	
					C = particle_vector[i].position +
						(RIGHT+UP) * 
						(particle_vector[i].size - size_increment);
					D = particle_vector[i].position -
						(RIGHT-UP) *
						(particle_vector[i].size - size_increment);
				
					glBegin(GL_TRIANGLE_STRIP);		// render billboard
						glTexCoord2d(1,1); 
						glVertex3f(C.x,C.y,C.z);
						glTexCoord2d(0,1); 
						glVertex3f(D.x,D.y,D.z);
						glTexCoord2d(1,0); 
						glVertex3f(B.x,B.y,B.z);
						glTexCoord2d(0,0); 
						glVertex3f(A.x,A.y,A.z);
					glEnd();

					glPopMatrix();
				}
			} // end for
		} // end if
	} // end for
}

//-------------------------------------------------------------
// rotator_system::reset() - resets to simple system
//-------------------------------------------------------------
void rotator_system::reset(int index)
{
	// reset CPPN
	CPPN_VECTOR[index].reset(4,6);			
}

#endif
