//-------------------------------------------------------------------------------
// texture.h - texture class and loading functions
//-------------------------------------------------------------------------------
#ifndef TEXTURE_H
#define TEXTURE_H

//-------------------------------------------------------------------------------
// declarations
//-------------------------------------------------------------------------------
AUX_RGBImageRec* LoadBMP(const char *Filename);

//-------------------------------------------------------------------------------
// Texture Class - texture class with filename and texture data
//-------------------------------------------------------------------------------
class texture
{
	public:

	string filename;
	GLuint data[1];

	texture();
	~texture();
	void load(string file);
};

texture::texture()
{
	filename = "null";
	data[0]=0;
}

texture::~texture()
{}

void texture::load(string file)
{
	AUX_RGBImageRec *TextureImage[1];						
	memset(TextureImage,0,sizeof(void *)*1);

	// save file name
	filename = file;

	// load data and filter
	if (TextureImage[0]=LoadBMP(file.c_str()))		
	{
		glGenTextures(1, &data[0]);					
		glBindTexture(GL_TEXTURE_2D, data[0]);		
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D,
			         0,
					 3,
					 TextureImage[0]->sizeX,
					 TextureImage[0]->sizeY,
					 0, 
					 GL_RGB,
					 GL_UNSIGNED_BYTE,
					 TextureImage[0]->data);
	}

	// If Texture Exists
	if (TextureImage[0])								
	{
		// If Texture Image Exists
		if (TextureImage[0]->data)						
		{	
			// Free The Texture Image Memory
			free(TextureImage[0]->data);				
		}
		// Free The Image Structure
		free(TextureImage[0]);						
	}
}

//-------------------------------------------------------------------------------
// LoadBMP -  Loads A Bitmap Image
//-------------------------------------------------------------------------------
AUX_RGBImageRec* LoadBMP(const char *Filename)		
{
	FILE *File=NULL;						// File Handle

	if (!Filename)							// Make Sure A Filename Was Given
	{
		return NULL;						// If Not Return NULL
	}

	File=fopen(Filename,"r");				// Check To See If The File Exists

	if (File)								// Does The File Exist?
	{
		fclose(File);						// Close The Handle
		return auxDIBImageLoad(Filename);	// Load The Bitmap And Return A Pointer
	}

	return NULL;							// If Load Failed Return NULL
}

#endif